@@ -16,8 +16,12 @@
 
 char *getSourceDateString(void);
 char *getProgramTitleString(void);
+char *getProgramRealVersionString(void);
 char *getProgramVersionString(void);
 char *getProgramInitString(void);
+char *getConfigProgramTitleString(void);
+char *getConfigProgramCopyrightString(void);
+char *getConfigProgramCompanyString(void);
 char *getWindowTitleString(void);
 
-#endif	/* CONFIG_H */
+#endif	// CONFIG_H
